<?php

use Imoje\Payment\Api;
use Imoje\Payment\Paywall;
use Imoje\Payment\Util;

include_once(Mage::getConfig()->getModuleDir('helper', 'Imoje') . '/Helper/Payment-core/src/Api.php');
include_once(Mage::getConfig()->getModuleDir('helper', 'Imoje') . '/Helper/Payment-core/src/Paywall.php');
include_once(Mage::getConfig()->getModuleDir('helper', 'Imoje') . '/Helper/Payment-core/src/Util.php');
include_once(Mage::getConfig()->getModuleDir('helper', 'Imoje') . '/Helper/Payment-core/src/CartData.php');

/**
 * Class Imoje_Model_Abstract
 */
abstract class Imoje_Model_Abstract extends Mage_Payment_Model_Method_Abstract
{
	protected $_canUseCheckout = true;
	protected $_canVoid = true;
	protected $_isInitializeNeeded = true;
	protected $_isGateway = true;
	protected $_canOrder = true;
	protected $_canReviewPayment = true;
	protected $_canRefund = true;

	/**
	 * Refund payment
	 *
	 * @param Varien_Object $payment
	 * @param float         $amount
	 *
	 * @return $this
	 * @throws Mage_Core_Exception
	 */
	public function refund(Varien_Object $payment, $amount)
	{

		$api = new Api(
			Mage::getStoreConfig('payment/imoje/authorization_token'),
			Mage::getStoreConfig('payment/imoje/merchant_id'),
			Mage::getStoreConfig('payment/imoje/service_id')
		);

		$refund = $api->createRefund(
			$api->prepareRefundData(
				Util::convertAmountToFractional($amount)
			),
			$payment->getParentTransactionId()
		);

		if($refund['success'] && $refund['body']['transaction']['status'] === 'settled') {

			return $this;
		}

		Mage::throwException(Mage::helper('imoje')->__('The refund action is not available at this moment. Try again later.'));
	}

	/**
	 * @return string
	 */
	public function getOrderPlaceRedirectUrl()
	{
		return Mage::getUrl('imoje/processing/redirect');
	}

	/**
	 * @param string $currencyCode
	 *
	 * @return bool
	 */
	public function canUseForCurrency($currencyCode)
	{

		return Util::canUseForCurrency(
				strtolower($currencyCode),
				explode(
					',',
					strtolower(
						str_replace(' ', '', Mage::getStoreConfig('payment/imoje/currency'))
					)
				)
			)
			&& Mage::getStoreConfig('payment/imoje/service_key')
			&& Mage::getStoreConfig('payment/imoje/service_id')
			&& Mage::getStoreConfig('payment/imoje/merchant_id');
	}

	/**
	 * @return string
	 */
	public function getOrderId()
	{
		return $this->getOrder()->getRealOrderId();
	}

	/**
	 * @return Mage_Sales_Model_Order
	 */
	public function getOrder()
	{
		if(!$this->_order) {
			$this->_order = $this->getInfoInstance()->getOrder();
		}

		return $this->_order;
	}

	/**
	 * @return array
	 */
	abstract public function getRedirectionForm();

	/**
	 * @param Mage_Sales_Model_Order $order
	 *
	 * @return mixed
	 */
	abstract public function getCart($order);
}
