<?php

use Imoje\Payment\CartData;
use Imoje\Payment\Paywall;
use Imoje\Payment\Util;

/**
 * Class Imoje_Model_Imoje
 */
class Imoje_Model_Imoje extends Imoje_Model_Abstract
{
	const CODE = 'imoje';

	/**
	 * @var string Payment method code
	 */
	protected $_code = self::CODE;

	/**
	 * @var string Payment form block path
	 */
	protected $_formBlockType = 'imoje/form_imoje';

	/**
	 * @return string
	 */
	public function getRedirectionForm()
	{

		$order = $this->getOrder();
		$billingAddress = $order->getBillingAddress();

		$paywall = new Paywall(
			(Mage::getStoreConfig('payment/imoje/sandbox') == 1
				? Util::ENVIRONMENT_SANDBOX
				: Util::ENVIRONMENT_PRODUCTION),
			Mage::getStoreConfig('payment/imoje/service_key'),
			Mage::getStoreConfig('payment/imoje/service_id'),
			Mage::getStoreConfig('payment/imoje/merchant_id'));

		return $paywall->buildOrderForm(
			Paywall::prepareData(
				Util::convertAmountToFractional($order->getGrandTotal()),
				$order->getOrderCurrencyCode(),
				$order->getRealOrderId(),
				$billingAddress->getFirstname(),
				$billingAddress->getLastname(),
				null,
				$billingAddress->getEmail(),
				$billingAddress->getTelephone(),
				Mage::getUrl('checkout/onepage/success'),
				Mage::getUrl('checkout/onepage/failure'),
				Mage::getUrl('checkout/onepage/failure'),
				Imoje_Helper_Data::VERSION . ';magento_' . Mage::getVersion(),
				$this->getCart($order)
			),
			Mage::helper('imoje')->__('Continue')
		);
	}

	/**
	 * @param Mage_Sales_Model_Order $order
	 *
	 * @return string
	 */
	public function getCart($order)
	{

		$cartData = $this->getCartData($order);

		$previousOrders = $this->getPreviousOrders($order->getBillingAddress()->getEmail());

		foreach($previousOrders as $previousOrder) {
			$cartData->addPrevious($previousOrder);
		}

		return $cartData->prepareCartData();
	}

	/**
	 * @param Mage_Sales_Model_Order $order
	 *
	 * @return CartData
	 */
	public function getCartData($order)
	{

		$quote = Mage::getSingleton('checkout/session')->getQuote();
		$store = $quote->getStore();
		$taxCalculation = Mage::getModel('tax/calculation');
		$request = $taxCalculation->getRateRequest(null, null, null, $store);

		$cartData = new CartData();

		foreach($order->getAllItems() as $item) {

			$cartData->addItem(
				$item->getId(),
				(float) $item->getTaxPercent(),
				$item->getName(),
				Util::convertAmountToFractional($item->getPriceInclTax()),
				(float) $item->getQtyOrdered()
			);
		}

		$cartData->setCreatedAt(Mage::getModel('core/date')->timestamp($order->getCreatedAt()));
		$cartData->setAmount(Util::convertAmountToFractional($order->getGrandTotal()));

		$addressBilling = $order->getBillingAddress();
		$addressDelivery = $addressBilling;

		if(!$order->getIsVirtual()){
			$addressDelivery = $order->getShippingAddress();
			$cartData->setShipping(
				(float) $taxCalculation->getRate($request->setProductClassId(Mage::getStoreConfig('tax/classes/shipping_tax_class', $store))),
				$order->getShippingDescription(),
				Util::convertAmountToFractional($order->getShippingInclTax())
			);
		}

		$cartData->setAddressBilling(
			$addressBilling->getCity(),
			$addressBilling->getFirstname() . ' ' . $addressBilling->getLastname(),
			$addressBilling->getTelephone(),
			$addressBilling->getStreet(1) . (empty($addressBilling->getStreet(2))
				? ''
				: '' . $addressBilling->getStreet(2)),
			$addressBilling->getCountryId(),
			$addressBilling->getPostcode()
		);

		$cartData->setAddressDelivery(
			$addressDelivery->getCity(),
			$addressDelivery->getFirstname() . ' ' . $addressDelivery->getLastname(),
			$addressDelivery->getTelephone(),
			$addressDelivery->getStreet(1) . (empty($addressDelivery->getStreet(2))
				? ''
				: '' . $addressDelivery->getStreet(2)),
			$addressDelivery->getCountryId(),
			$addressDelivery->getPostcode()
		);

		if($order->getDiscountAmount() < 0) {
			$cartData->setDiscount(
				Util::convertAmountToFractional(abs($order->getDiscountAmount())),
				'discount',
				0
			);
		}

		return $cartData;
	}

	/**
	 * @param string $customerEmail
	 *
	 * @return CartData[]
	 */
	public function getPreviousOrders($customerEmail)
	{

		$orderCollection = Mage::getModel('sales/order')->getCollection();
		$orderCollection->addFieldToFilter('customer_email', $customerEmail);
		$orderCollection->addFieldToFilter('status', [
			'complete',
			'processing',
		]);

		$previousOrders = [];

		foreach($orderCollection as $order) {

			$previousOrders[] = $this->getCartData($order);
		}

		return $previousOrders;
	}
}
