<?php

use Imoje\Payment\Util;

/**
 * Class Imoje_Model_Paylater
 */
class Imoje_Model_Paylater extends Imoje_Model_Abstract
{

	const CODE = 'paylater';

	/**
	 * @var string Payment method code
	 */
	protected $_code = self::CODE;

	/**
	 * @var string Payment form block path
	 */
	protected $_formBlockType = 'imoje/form_paylater';

	/**
	 * @return string
	 */
	public function getOrderPlaceRedirectUrl()
	{
		return Mage::getUrl('imoje/processing/redirect');
	}

	/**
	 * @return string|void
	 * @throws Exception
	 */
	public function getRedirectionForm()
	{

		$environment = Mage::getStoreConfig('payment/imoje/sandbox') == 1
			? Util::ENVIRONMENT_SANDBOX
			: Util::ENVIRONMENT_PRODUCTION;

		$api = new \Imoje\Payment\Api(
			Mage::getStoreConfig('payment/imoje/authorization_token'),
			Mage::getStoreConfig('payment/imoje/merchant_id'),
			Mage::getStoreConfig('payment/imoje/service_id'),
			$environment
		);

		$order = $this->getOrder();
		$billingAddress = $order->getBillingAddress();

		$transaction = $api->createTransaction(
			$api->prepareData(
				Util::convertAmountToFractional($this->getOrder()->getGrandTotal()),
				$order->getOrderCurrencyCode(),
				$order->getRealOrderId(),
				Util::getPaymentMethod('paylater'),
				Util::getPaymentMethodCode('paylater'),
				Mage::getUrl('checkout/onepage/success'),
				Mage::getUrl('checkout/onepage/failure'),
				$billingAddress->getFirstname(),
				$billingAddress->getLastname(),
				$billingAddress->getEmail(),
				\Imoje\Payment\Api::TRANSACTION_TYPE_SALE,
				'',
				'',
				$this->getAddress($order)
			)
		);

		if(!$transaction['success']) {
			Mage::app()->getFrontController()->getResponse()->setRedirect(Mage::getBaseUrl());

			return;
		}

		return $api->buildOrderForm(
			$transaction,
			Mage::helper('imoje')->__('Continue')
		);
	}

	/**
	 * @param Mage_Sales_Model_Order $order
	 *
	 * @return array
	 */
	private function getAddress($order)
	{
		$addressBilling = $order->getBillingAddress();

		$address = [
			'billing' => \Imoje\Payment\Api::prepareAddressData(
				$addressBilling->getFirstname(),
				$addressBilling->getLastname(),
				$addressBilling->getStreet(1) . (empty($addressBilling->getStreet(2))
					? ''
					: '' . $addressBilling->getStreet(2)),
				$addressBilling->getCity(),
				'',
				$addressBilling->getPostcode(),
				$addressBilling->getCountryId()
			),
		];

		if($order->hasShipments()) {
			$addressDelivery = $order->getShippingAddress();

			$address['shipping'] = \Imoje\Payment\Api::prepareAddressData(
				$addressDelivery->getFirstname(),
				$addressDelivery->getLastname(),
				$addressDelivery->getStreet(1) . (empty($addressDelivery->getStreet(2))
					? ''
					: '' . $addressDelivery->getStreet(2)),
				$addressDelivery->getCity(),
				'',
				$addressDelivery->getPostcode(),
				$addressDelivery->getCountryId()
			);
		}

		return $address;
	}

	/**
	 * @param Mage_Sales_Model_Order $order
	 *
	 * @return string
	 */
	public function getCart($order)
	{

		return '';
	}
}
