<?php

use Imoje\Payment\Notification;

include_once(Mage::getConfig()->getModuleDir('model', 'Imoje') . '/Helper/Payment-core/vendor/autoload.php');
include_once(Mage::getConfig()->getModuleDir('model', 'Imoje') . '/Helper/Payment-core/src/Util.php');
include_once(Mage::getConfig()->getModuleDir('model', 'Imoje') . '/Helper/Payment-core/src/Configuration.php');

/**
 * Class Imoje_NotificationController
 */
class Imoje_NotificationController extends Mage_Core_Controller_Front_Action
{

	/**
	 * Verify notification and change status
	 */
	public function processAction()
	{

		$notification = new Notification(
			Mage::getStoreConfig('payment/imoje/service_id'),
			Mage::getStoreConfig('payment/imoje/service_key')
		);

		$result_check_request_notification = $notification->checkRequest();

		$debug_mode = Mage::getStoreConfig('payment/imoje/debug_mode');

		if(is_int($result_check_request_notification)) {
			echo Notification::formatResponse(Notification::NS_ERROR, $result_check_request_notification, $debug_mode);
			exit();
		}

		$order = Mage::getModel("sales/order")->loadByIncrementId($result_check_request_notification['transaction']['orderId']);

		if(!$order) {
			echo Notification::formatResponse(Notification::NS_ERROR, Notification::NC_ORDER_NOT_FOUND, $debug_mode);
			exit();
		}

		if($order->getState() === Mage_Sales_Model_Order::STATE_PROCESSING) {
			echo Notification::formatResponse(Notification::NS_OK, Notification::NC_ORDER_STATUS_NOT_CHANGED, $debug_mode);
			exit();
		}

		$payment = $order->getPayment();
		$transactionReference = $result_check_request_notification['transaction']['id'];

		// Handle transaction states
		switch($result_check_request_notification['transaction']['status']) {
			case 'settled':
				$payment->setTransactionId($transactionReference)
					->setPreparedMessage(Mage::helper('core')->__('The transaction completed successfully'))
					->setCurrencyCode($payment->getOrder()->getBaseCurrencyCode())
					->setIsTransactionApproved(true)
					->setIsTransactionClosed(true)
					->registerCaptureNotification($order->getTotalDue(), true)
					->save();

				$order->save();

				if($invoice = $payment->getCreatedInvoice()) {
					$comment = Mage::helper('imoje')->__('Notified customer about invoice #%s.', $invoice->getIncrementId());
					$order->queueNewOrderEmail()
						->addStatusHistoryComment($comment)
						->setIsCustomerNotified(true)
						->save();
				}
				echo Notification::formatResponse(Notification::NS_OK, Notification::NC_OK, $debug_mode);
				die();
			case 'rejected':
				$comment = Mage::helper('imoje')->__('The payment has been rejected.');

				$payment->setTransactionId($transactionReference)
					->setPreparedMessage($comment)
					->save();

				$order->setState(Mage_Sales_Model_Order::STATE_PAYMENT_REVIEW, true, $comment)
					->save();
				echo Notification::formatResponse(Notification::NS_OK, Notification::NC_OK, $debug_mode);
				die();
			default:
				echo Notification::formatResponse(Notification::NS_OK, Notification::NC_ORDER_STATUS_NOT_CHANGED, $debug_mode);
				die();
		}
	}
}
